@@ -38,6 +38,13 @@
 
 #include "tracker-db-interface-sqlite.h"
 
+typedef struct {
+	TrackerDBStatement *head;
+	TrackerDBStatement *tail;
+	guint size;
+	guint max;
+} TrackerDBStatementLru;
+
 struct TrackerDBInterface {
 	GObject parent_instance;
 
@@ -52,9 +59,16 @@ struct TrackerDBInterface {
 
 	guint ro : 1;
 #if HAVE_TRACKER_FTS
-	guint fts_initialized : 1;
+	TrackerFts *fts;
 #endif
 	GCancellable *cancellable;
+
+	/* async operation pending */
+	gboolean pending;
+	GAsyncReadyCallback outstanding_callback;
+
+	TrackerDBStatementLru select_stmt_lru;
+	TrackerDBStatementLru update_stmt_lru;
 };
 
 struct TrackerDBInterfaceClass {
@@ -66,6 +80,10 @@ struct TrackerDBCursor {
 	sqlite3_stmt *stmt;
 	TrackerDBStatement *ref_stmt;
 	gboolean finished;
+	TrackerPropertyType *types;
+	gint n_types;
+	gchar **variable_names;
+	gint n_variable_names;
 };
 
 struct TrackerDBCursorClass {
@@ -77,17 +95,28 @@ struct TrackerDBStatement {
 	TrackerDBInterface *db_interface;
 	sqlite3_stmt *stmt;
 	gboolean stmt_is_sunk;
+	TrackerDBStatement *next;
+	TrackerDBStatement *prev;
 };
 
 struct TrackerDBStatementClass {
 	GObjectClass parent_class;
 };
 
-static TrackerDBStatement       * tracker_db_statement_sqlite_new (TrackerDBInterface     *db_interface,
-                                                                   sqlite3_stmt           *sqlite_stmt);
-static TrackerDBCursor          * tracker_db_cursor_sqlite_new    (sqlite3_stmt           *sqlite_stmt,
-                                                                   TrackerDBStatement     *ref_stmt);
-static void tracker_db_statement_sqlite_reset (TrackerDBStatement *stmt);
+static TrackerDBStatement *tracker_db_statement_sqlite_new   (TrackerDBInterface   *db_interface,
+                                                              sqlite3_stmt         *sqlite_stmt);
+static void                tracker_db_statement_sqlite_reset (TrackerDBStatement   *stmt);
+static TrackerDBCursor    *tracker_db_cursor_sqlite_new      (sqlite3_stmt         *sqlite_stmt,
+                                                              TrackerDBStatement   *ref_stmt,
+                                                              TrackerPropertyType  *types,
+                                                              gint                  n_types,
+                                                              const gchar         **variable_names,
+                                                              gint                  n_variable_names);
+static gboolean            tracker_db_cursor_get_boolean     (TrackerSparqlCursor  *cursor,
+                                                              guint                 column);
+static gboolean            db_cursor_iter_next               (TrackerDBCursor      *cursor,
+                                                              GCancellable         *cancellable,
+                                                              GError              **error);
 
 enum {
 	PROP_0,
@@ -627,8 +656,8 @@ close_database (TrackerDBInterface *db_interface)
 	db_interface->function_data = NULL;
 
 #if HAVE_TRACKER_FTS
-	if (db_interface->fts_initialized) {
-		tracker_fts_shutdown (G_OBJECT (db_interface));
+	if (db_interface->fts) {
+		tracker_fts_free (db_interface->fts);
 	}
 #endif
 
@@ -641,13 +670,43 @@ tracker_db_interface_sqlite_fts_init (TrackerDBInterface *db_interface,
                                       gboolean            create)
 {
 #if HAVE_TRACKER_FTS
-	tracker_fts_init (db_interface->db, create, G_OBJECT (db_interface));
-	db_interface->fts_initialized = TRUE;
+	db_interface->fts = tracker_fts_new (db_interface->db, create);
 #else
 	g_message ("FTS support is disabled");
 #endif
 }
 
+#if HAVE_TRACKER_FTS
+int
+tracker_db_interface_sqlite_fts_update_init (TrackerDBInterface *db_interface,
+                                             int                 id)
+{
+	return tracker_fts_update_init (db_interface->fts, id);
+}
+
+int
+tracker_db_interface_sqlite_fts_update_text (TrackerDBInterface *db_interface,
+                                             int                 id,
+                                             int                 column_id,
+                                             const char         *text,
+                                             gboolean            limit_word_length)
+{
+	return tracker_fts_update_text (db_interface->fts, id, column_id, text, limit_word_length);
+}
+
+void
+tracker_db_interface_sqlite_fts_update_commit (TrackerDBInterface *db_interface)
+{
+	return tracker_fts_update_commit (db_interface->fts);
+}
+
+void
+tracker_db_interface_sqlite_fts_update_rollback (TrackerDBInterface *db_interface)
+{
+	return tracker_fts_update_rollback (db_interface->fts);
+}
+#endif
+
 static void
 tracker_db_interface_sqlite_finalize (GObject *object)
 {
@@ -768,27 +827,76 @@ add_row (TrackerDBResultSet *result_set,
 	}
 }
 
+void
+tracker_db_interface_set_max_stmt_cache_size (TrackerDBInterface         *db_interface,
+                                              TrackerDBStatementCacheType cache_type,
+                                              guint                       max_size)
+{
+	TrackerDBStatementLru *stmt_lru;
+
+	if (cache_type == TRACKER_DB_STATEMENT_CACHE_TYPE_UPDATE) {
+		stmt_lru = &db_interface->update_stmt_lru;
+	} else if (cache_type == TRACKER_DB_STATEMENT_CACHE_TYPE_SELECT) {
+		stmt_lru = &db_interface->select_stmt_lru;
+	} else {
+		return;
+	}
+
+	/* Must be larger than 2 to make sense (to have a tail and head) */
+	if (max_size > 2)
+		stmt_lru->max = max_size;
+	else
+		stmt_lru->max = 3;
+}
 
 TrackerDBStatement *
-tracker_db_interface_create_statement (TrackerDBInterface  *db_interface,
-                                       GError             **error,
-                                       const gchar         *query,
+tracker_db_interface_create_statement (TrackerDBInterface           *db_interface,
+                                       TrackerDBStatementCacheType   cache_type,
+                                       GError                      **error,
+                                       const gchar                  *query,
                                        ...)
 {
+	TrackerDBStatementLru *stmt_lru = NULL;
 	TrackerDBStatement *stmt;
 	va_list args;
 	gchar *full_query;
 
 	g_return_val_if_fail (TRACKER_IS_DB_INTERFACE (db_interface), NULL);
+	g_return_val_if_fail (!db_interface->pending, NULL);
 
 	va_start (args, query);
 	full_query = g_strdup_vprintf (query, args);
 	va_end (args);
 
-	stmt = g_hash_table_lookup (db_interface->dynamic_statements, full_query);
+	/* There are three kinds of queries: 
+	  * a) Cached queries: SELECT and UPDATE ones (cache_type)
+	  * b) Non-Cached queries: NONE ones (cache_type)
+	  * c) Forced Non-Cached: in case of a stmt being already in use, we can't
+	  *    reuse it (you can't use two different loops on a sqlite3_stmt, of
+	  *    course). This happens with recursive uses of a cursor, for example */
 
-	if (stmt && stmt->stmt_is_sunk) {
-		/* prepared statement is still in use, create new one */
+	if (cache_type != TRACKER_DB_STATEMENT_CACHE_TYPE_NONE) {
+		stmt = g_hash_table_lookup (db_interface->dynamic_statements, full_query);
+
+		if (stmt && stmt->stmt_is_sunk) {
+			/* c) Forced non-cached
+			 * prepared statement is still in use, create new uncached one */
+			stmt = NULL;
+			/* Make sure to set cache_type here, to ensure the right flow in the
+			 * LRU-cache below */
+			cache_type = TRACKER_DB_STATEMENT_CACHE_TYPE_NONE;
+		}
+
+		if (cache_type == TRACKER_DB_STATEMENT_CACHE_TYPE_UPDATE) {
+			/* a) Cached */
+			stmt_lru = &db_interface->update_stmt_lru;
+		} else {
+			/* a) Cached */
+			stmt_lru = &db_interface->select_stmt_lru;
+		}
+
+	} else {
+		/* b) Non-Cached */
 		stmt = NULL;
 	}
 
@@ -822,19 +930,91 @@ tracker_db_interface_create_statement (TrackerDBInterface  *db_interface,
 
 		stmt = tracker_db_statement_sqlite_new (db_interface, sqlite_stmt);
 
-		/* use replace instead of insert to make sure we store the string that
-		   belongs to the right sqlite statement to ensure the lifetime of the string
-		   matches the statement */
-		g_hash_table_replace (db_interface->dynamic_statements,
-		                      (gpointer) sqlite3_sql (sqlite_stmt),
-		                      stmt);
+		if (cache_type != TRACKER_DB_STATEMENT_CACHE_TYPE_NONE) {
+			/* use replace instead of insert to make sure we store the string that
+			   belongs to the right sqlite statement to ensure the lifetime of the string
+			   matches the statement */
+			g_hash_table_replace (db_interface->dynamic_statements,
+			                      (gpointer) sqlite3_sql (sqlite_stmt),
+			                      stmt);
+
+			/* So the ring looks a bit like this: *
+			 *                                    *
+			 *    .--tail  .--head                *
+			 *    |        |                      *
+			 *  [p-n] -> [p-n] -> [p-n] -> [p-n]  *
+			 *    ^                          |    *
+			 *    `- [n-p] <- [n-p] <--------'    *
+			 *                                    */
+
+			if (stmt_lru->size >= stmt_lru->max) {
+				TrackerDBStatement *new_head;
+
+				/* We reached max-size of the LRU stmt cache. Destroy current
+				 * least recently used (stmt_lru.head) and fix the ring. For
+				 * that we take out the current head, and close the ring.
+				 * Then we assign head->next as new head. */
+
+				new_head = stmt_lru->head->next;
+				g_hash_table_remove (db_interface->dynamic_statements,
+				                     (gpointer) sqlite3_sql (stmt_lru->head->stmt));
+				stmt_lru->size--;
+				stmt_lru->head = new_head;
+			} else {
+				if (stmt_lru->size == 0) {
+					stmt_lru->head = stmt;
+					stmt_lru->tail = stmt;
+				}
+			}
+
+			/* Set the current stmt (which is always new here) as the new tail
+			 * (new most recent used). We insert current stmt between head and
+			 * current tail, and we set tail to current stmt. */
+
+			stmt_lru->size++;
+			stmt->next = stmt_lru->head;
+			stmt_lru->head->prev = stmt;
+
+			stmt_lru->tail->next = stmt;
+			stmt->prev = stmt_lru->tail;
+			stmt_lru->tail = stmt;
+		}
 	} else {
 		tracker_db_statement_sqlite_reset (stmt);
+
+		if (cache_type != TRACKER_DB_STATEMENT_CACHE_TYPE_NONE) {
+			if (stmt == stmt_lru->head) {
+
+				/* Current stmt is least recently used, shift head and tail
+				 * of the ring to efficiently make it most recently used. */
+
+				stmt_lru->head = stmt_lru->head->next;
+				stmt_lru->tail = stmt_lru->tail->next;
+			} else if (stmt != stmt_lru->tail) {
+
+				/* Current statement isn't most recently used, make it most
+				 * recently used now (less efficient way than above). */
+
+				/* Take stmt out of the list and close the ring */
+				stmt->prev->next = stmt->next;
+				stmt->next->prev = stmt->prev;
+
+				/* Put stmt as tail (most recent used) */
+				stmt->next = stmt_lru->head;
+				stmt_lru->head->prev = stmt;
+				stmt->prev = stmt_lru->tail;
+				stmt_lru->tail->next = stmt;
+				stmt_lru->tail = stmt;
+			}
+
+			/* if (stmt == tail), it's already the most recently used in the
+			 * ring, so in this case we do nothing of course */
+		}
 	}
 
 	g_free (full_query);
 
-	return g_object_ref (stmt);
+	return (cache_type != TRACKER_DB_STATEMENT_CACHE_TYPE_NONE) ? g_object_ref (stmt) : stmt;
 }
 
 static TrackerDBResultSet *
@@ -1030,8 +1210,8 @@ tracker_db_statement_class_init (TrackerDBStatementClass *class)
 }
 
 static TrackerDBStatement *
-tracker_db_statement_sqlite_new (TrackerDBInterface       *db_interface,
-                                 sqlite3_stmt             *sqlite_stmt)
+tracker_db_statement_sqlite_new (TrackerDBInterface *db_interface,
+                                 sqlite3_stmt       *sqlite_stmt)
 {
 	TrackerDBStatement *stmt;
 
@@ -1048,9 +1228,17 @@ static void
 tracker_db_cursor_finalize (GObject *object)
 {
 	TrackerDBCursor *cursor;
+	int i;
 
 	cursor = TRACKER_DB_CURSOR (object);
 
+	g_free (cursor->types);
+
+	for (i = 0; i < cursor->n_variable_names; i++) {
+		g_free (cursor->variable_names[i]);
+	}
+	g_free (cursor->variable_names);
+
 	if (cursor->ref_stmt) {
 		cursor->ref_stmt->stmt_is_sunk = FALSE;
 		tracker_db_statement_sqlite_reset (cursor->ref_stmt);
@@ -1069,10 +1257,11 @@ tracker_db_cursor_iter_next_thread (GSimpleAsyncResult *res,
 {
 	/* run in thread */
 
+	TrackerDBCursor *cursor = TRACKER_DB_CURSOR (object);
 	GError *error = NULL;
 	gboolean result;
 
-	result = tracker_db_cursor_iter_next (TRACKER_DB_CURSOR (object), cancellable, &error);
+	result = db_cursor_iter_next (cursor, cancellable, &error);
 	if (error) {
 		g_simple_async_result_set_from_error (res, error);
 	} else {
@@ -1080,6 +1269,24 @@ tracker_db_cursor_iter_next_thread (GSimpleAsyncResult *res,
 	}
 }
 
+static void
+async_ready_callback_wrapper (GObject      *source_object,
+                              GAsyncResult *res,
+                              gpointer      user_data)
+{
+	TrackerDBCursor *cursor = TRACKER_DB_CURSOR (source_object);
+	TrackerDBInterface *iface = cursor->ref_stmt->db_interface;
+
+	iface->pending = FALSE;
+	if (iface->outstanding_callback) {
+		GAsyncReadyCallback callback = iface->outstanding_callback;
+
+		iface->outstanding_callback = NULL;
+		(callback) (source_object, res, user_data);
+	}
+	g_object_unref (cursor);
+}
+
 static void
 tracker_db_cursor_iter_next_async (TrackerDBCursor     *cursor,
                                    GCancellable        *cancellable,
@@ -1088,7 +1295,11 @@ tracker_db_cursor_iter_next_async (TrackerDBCursor     *cursor,
 {
 	GSimpleAsyncResult *res;
 
-	res = g_simple_async_result_new (G_OBJECT (cursor), callback, user_data, tracker_db_cursor_iter_next_async);
+	g_return_if_fail (!cursor->ref_stmt->db_interface->pending);
+	cursor->ref_stmt->db_interface->pending = TRUE;
+	cursor->ref_stmt->db_interface->outstanding_callback = callback;
+
+	res = g_simple_async_result_new (g_object_ref (cursor), async_ready_callback_wrapper, user_data, tracker_db_cursor_iter_next_async);
 	g_simple_async_result_run_in_thread (res, tracker_db_cursor_iter_next_thread, 0, cancellable);
 }
 
@@ -1111,17 +1322,27 @@ tracker_db_cursor_class_init (TrackerDBCursorClass *class)
 
 	object_class->finalize = tracker_db_cursor_finalize;
 
+	sparql_cursor_class->get_value_type = (TrackerSparqlValueType (*) (TrackerSparqlCursor *, gint)) tracker_db_cursor_get_value_type;
+	sparql_cursor_class->get_variable_name = (const gchar * (*) (TrackerSparqlCursor *, gint)) tracker_db_cursor_get_variable_name;
 	sparql_cursor_class->get_n_columns = (gint (*) (TrackerSparqlCursor *)) tracker_db_cursor_get_n_columns;
 	sparql_cursor_class->get_string = (const gchar * (*) (TrackerSparqlCursor *, gint, glong*)) tracker_db_cursor_get_string;
 	sparql_cursor_class->next = (gboolean (*) (TrackerSparqlCursor *, GCancellable *, GError **)) tracker_db_cursor_iter_next;
 	sparql_cursor_class->next_async = (void (*) (TrackerSparqlCursor *, GCancellable *, GAsyncReadyCallback, gpointer)) tracker_db_cursor_iter_next_async;
 	sparql_cursor_class->next_finish = (gboolean (*) (TrackerSparqlCursor *, GAsyncResult *, GError **)) tracker_db_cursor_iter_next_finish;
 	sparql_cursor_class->rewind = (void (*) (TrackerSparqlCursor *)) tracker_db_cursor_rewind;
+
+	sparql_cursor_class->get_integer = (gint64 (*) (TrackerSparqlCursor *, gint)) tracker_db_cursor_get_int;
+	sparql_cursor_class->get_double = (gdouble (*) (TrackerSparqlCursor *, gint)) tracker_db_cursor_get_double;
+	sparql_cursor_class->get_boolean = (gboolean (*) (TrackerSparqlCursor *, gint)) tracker_db_cursor_get_boolean;
 }
 
 static TrackerDBCursor *
-tracker_db_cursor_sqlite_new (sqlite3_stmt              *sqlite_stmt,
-                              TrackerDBStatement        *ref_stmt)
+tracker_db_cursor_sqlite_new (sqlite3_stmt        *sqlite_stmt,
+                              TrackerDBStatement  *ref_stmt,
+                              TrackerPropertyType *types,
+                              gint                 n_types,
+                              const gchar        **variable_names,
+                              gint                 n_variable_names)
 {
 	TrackerDBCursor *cursor;
 
@@ -1137,13 +1358,33 @@ tracker_db_cursor_sqlite_new (sqlite3_stmt              *sqlite_stmt,
 		cursor->ref_stmt = NULL;
 	}
 
+	if (types) {
+		gint i;
+
+		cursor->types = g_new (TrackerPropertyType, n_types);
+		cursor->n_types = n_types;
+		for (i = 0; i < n_types; i++) {
+			cursor->types[i] = types[i];
+		}
+	}
+
+	if (variable_names) {
+		gint i;
+
+		cursor->variable_names = g_new (gchar *, n_variable_names);
+		cursor->n_variable_names = n_variable_names;
+		for (i = 0; i < n_variable_names; i++) {
+			cursor->variable_names[i] = g_strdup (variable_names[i]);
+		}
+	}
+
 	return cursor;
 }
 
 void
-tracker_db_statement_bind_double (TrackerDBStatement      *stmt,
-                                  int                      index,
-                                  double                   value)
+tracker_db_statement_bind_double (TrackerDBStatement *stmt,
+                                  int                 index,
+                                  double              value)
 {
 	g_return_if_fail (TRACKER_IS_DB_STATEMENT (stmt));
 
@@ -1153,9 +1394,9 @@ tracker_db_statement_bind_double (TrackerDBStatement      *stmt,
 }
 
 void
-tracker_db_statement_bind_int (TrackerDBStatement       *stmt,
-                               int                       index,
-                               gint64                    value)
+tracker_db_statement_bind_int (TrackerDBStatement *stmt,
+                               int                 index,
+                               gint64              value)
 {
 	g_return_if_fail (TRACKER_IS_DB_STATEMENT (stmt));
 
@@ -1165,8 +1406,8 @@ tracker_db_statement_bind_int (TrackerDBStatement       *stmt,
 }
 
 void
-tracker_db_statement_bind_null (TrackerDBStatement        *stmt,
-                                int                        index)
+tracker_db_statement_bind_null (TrackerDBStatement *stmt,
+                                int                 index)
 {
 	g_return_if_fail (TRACKER_IS_DB_STATEMENT (stmt));
 
@@ -1176,9 +1417,9 @@ tracker_db_statement_bind_null (TrackerDBStatement        *stmt,
 }
 
 void
-tracker_db_statement_bind_text (TrackerDBStatement        *stmt,
-                                int                        index,
-                                const gchar               *value)
+tracker_db_statement_bind_text (TrackerDBStatement *stmt,
+                                int                 index,
+                                const gchar        *value)
 {
 	g_return_if_fail (TRACKER_IS_DB_STATEMENT (stmt));
 
@@ -1191,6 +1432,7 @@ void
 tracker_db_cursor_rewind (TrackerDBCursor *cursor)
 {
 	g_return_if_fail (TRACKER_IS_DB_CURSOR (cursor));
+	g_return_if_fail (!cursor->ref_stmt->db_interface->pending);
 
 	sqlite3_reset (cursor->stmt);
 	cursor->finished = FALSE;
@@ -1200,6 +1442,17 @@ gboolean
 tracker_db_cursor_iter_next (TrackerDBCursor *cursor,
                              GCancellable    *cancellable,
                              GError         **error)
+{
+	g_return_val_if_fail (!cursor->ref_stmt->db_interface->pending, FALSE);
+
+	return db_cursor_iter_next (cursor, cancellable, error);
+}
+
+
+static gboolean
+db_cursor_iter_next (TrackerDBCursor *cursor,
+                     GCancellable    *cancellable,
+                     GError         **error)
 {
 	TrackerDBStatement *stmt = cursor->ref_stmt;
 	TrackerDBInterface *iface = stmt->db_interface;
@@ -1243,7 +1496,9 @@ tracker_db_cursor_get_n_columns (TrackerDBCursor *cursor)
 }
 
 void
-tracker_db_cursor_get_value (TrackerDBCursor *cursor,  guint column, GValue *value)
+tracker_db_cursor_get_value (TrackerDBCursor *cursor,
+                             guint            column,
+                             GValue          *value)
 {
 	gint col_type;
 
@@ -1272,20 +1527,72 @@ tracker_db_cursor_get_value (TrackerDBCursor *cursor,  guint column, GValue *val
 }
 
 gint64
-tracker_db_cursor_get_int (TrackerDBCursor *cursor,  guint column)
+tracker_db_cursor_get_int (TrackerDBCursor *cursor,
+                           guint            column)
 {
 	return (gint64) sqlite3_column_int64 (cursor->stmt, column);
 }
 
 gdouble
-tracker_db_cursor_get_double (TrackerDBCursor *cursor,  guint column)
+tracker_db_cursor_get_double (TrackerDBCursor *cursor,
+                              guint            column)
 {
 	return (gdouble) sqlite3_column_double (cursor->stmt, column);
 }
 
+static gboolean
+tracker_db_cursor_get_boolean (TrackerSparqlCursor *sparql_cursor,
+                               guint                column)
+{
+	TrackerDBCursor *cursor = (TrackerDBCursor *) sparql_cursor;
+	return (g_strcmp0 (sqlite3_column_text (cursor->stmt, column), "true") == 0);
+}
+
+TrackerSparqlValueType
+tracker_db_cursor_get_value_type (TrackerDBCursor *cursor,
+                                  guint            column)
+{
+	gint n_columns = sqlite3_column_count (cursor->stmt);
+
+	g_return_val_if_fail (column < n_columns, TRACKER_SPARQL_VALUE_TYPE_UNBOUND);
+
+	if (sqlite3_column_type (cursor->stmt, column) == SQLITE_NULL) {
+		return TRACKER_SPARQL_VALUE_TYPE_UNBOUND;
+	} else if (column < cursor->n_types) {
+		switch (cursor->types[column]) {
+		case TRACKER_PROPERTY_TYPE_RESOURCE:
+			return TRACKER_SPARQL_VALUE_TYPE_URI;
+		case TRACKER_PROPERTY_TYPE_INTEGER:
+			return TRACKER_SPARQL_VALUE_TYPE_INTEGER;
+		case TRACKER_PROPERTY_TYPE_DOUBLE:
+			return TRACKER_SPARQL_VALUE_TYPE_DOUBLE;
+		case TRACKER_PROPERTY_TYPE_DATETIME:
+			return TRACKER_SPARQL_VALUE_TYPE_DATETIME;
+		case TRACKER_PROPERTY_TYPE_BOOLEAN:
+			return TRACKER_SPARQL_VALUE_TYPE_BOOLEAN;
+		default:
+			return TRACKER_SPARQL_VALUE_TYPE_STRING;
+		}
+	} else {
+		return TRACKER_SPARQL_VALUE_TYPE_STRING;
+	}
+}
 
 const gchar*
-tracker_db_cursor_get_string (TrackerDBCursor *cursor,  guint column, glong *length)
+tracker_db_cursor_get_variable_name (TrackerDBCursor *cursor,
+                                     guint            column)
+{
+	if (column < cursor->n_variable_names) {
+		return cursor->variable_names[column];
+	} else {
+		return sqlite3_column_name (cursor->stmt, column);
+	}
+}
+
+const gchar*
+tracker_db_cursor_get_string (TrackerDBCursor *cursor,
+                              guint            column,
+                              glong           *length)
 {
 	if (length) {
 		sqlite3_value *val = sqlite3_column_value (cursor->stmt, column);
@@ -1297,10 +1604,9 @@ tracker_db_cursor_get_string (TrackerDBCursor *cursor,  guint column, glong *len
 	}
 }
 
-
 TrackerDBResultSet *
-tracker_db_statement_execute (TrackerDBStatement          *stmt,
-                              GError                     **error)
+tracker_db_statement_execute (TrackerDBStatement  *stmt,
+                              GError             **error)
 {
 	g_return_val_if_fail (!stmt->stmt_is_sunk, NULL);
 
@@ -1308,14 +1614,26 @@ tracker_db_statement_execute (TrackerDBStatement          *stmt,
 }
 
 TrackerDBCursor *
-tracker_db_statement_start_cursor (TrackerDBStatement             *stmt,
-                                   GError                        **error)
+tracker_db_statement_start_cursor (TrackerDBStatement  *stmt,
+                                   GError             **error)
 {
 	g_return_val_if_fail (!stmt->stmt_is_sunk, NULL);
 
-	return tracker_db_cursor_sqlite_new (stmt->stmt, stmt);
+	return tracker_db_cursor_sqlite_new (stmt->stmt, stmt, NULL, 0, NULL, 0);
 }
 
+TrackerDBCursor *
+tracker_db_statement_start_sparql_cursor (TrackerDBStatement   *stmt,
+                                          TrackerPropertyType  *types,
+                                          gint                  n_types,
+                                          const gchar         **variable_names,
+                                          gint                  n_variable_names,
+                                          GError              **error)
+{
+	g_return_val_if_fail (!stmt->stmt_is_sunk, NULL);
+
+	return tracker_db_cursor_sqlite_new (stmt->stmt, stmt, types, n_types, variable_names, n_variable_names);
+}
 
 static void
 tracker_db_statement_init (TrackerDBStatement *stmt)
