@@ -56,8 +56,9 @@
 #define TRACKER_DB_PAGE_SIZE_DONT_SET -1
 
 /* Set current database version we are working with */
-#define TRACKER_DB_VERSION_NOW        TRACKER_DB_VERSION_0_9_16
+#define TRACKER_DB_VERSION_NOW        TRACKER_DB_VERSION_0_9_21
 #define TRACKER_DB_VERSION_FILE       "db-version.txt"
+#define TRACKER_DB_LOCALE_FILE        "db-locale.txt"
 
 #define IN_USE_FILENAME               ".meta.isrunning"
 
@@ -90,7 +91,9 @@ typedef enum {
 	TRACKER_DB_VERSION_0_9_0,   /* unstable release */
 	TRACKER_DB_VERSION_0_9_8,   /* affiliation cardinality + volumes */
 	TRACKER_DB_VERSION_0_9_15,  /* mtp:hidden */
-	TRACKER_DB_VERSION_0_9_16   /* Fix for NB#184823 */
+	TRACKER_DB_VERSION_0_9_16,  /* Fix for NB#184823 */
+	TRACKER_DB_VERSION_0_9_19,  /* collation */
+	TRACKER_DB_VERSION_0_9_21   /* Fix for NB#186055 */
 } TrackerDBVersion;
 
 typedef struct {
@@ -130,17 +133,6 @@ static TrackerDBDefinition dbs[] = {
 	  FALSE,
 	  FALSE,
 	  0 },
-	{ TRACKER_DB_FULLTEXT,
-	  TRACKER_DB_LOCATION_DATA_DIR,
-	  NULL,
-	  "fulltext.db",
-	  "fulltext",
-	  NULL,
-	  512,
-	  TRACKER_DB_PAGE_SIZE_DONT_SET,
-	  FALSE,
-	  TRUE,
-	  0 },
 };
 
 static gboolean            db_exec_no_reply    (TrackerDBInterface *iface,
@@ -149,6 +141,8 @@ static gboolean            db_exec_no_reply    (TrackerDBInterface *iface,
 static TrackerDBInterface *db_interface_create (TrackerDB           db);
 static TrackerDBInterface *tracker_db_manager_get_db_interfaces     (gint num, ...);
 static TrackerDBInterface *tracker_db_manager_get_db_interfaces_ro  (gint num, ...);
+static void                db_remove_version_file (void);
+static void                db_remove_locale_file  (void);
 
 static gboolean              initialized;
 static gboolean              locations_initialized;
@@ -156,11 +150,11 @@ static gchar                *sql_dir;
 static gchar                *data_dir = NULL;
 static gchar                *user_data_dir = NULL;
 static gchar                *sys_tmp_dir = NULL;
+static gchar                *in_use_filename = NULL;
 static gpointer              db_type_enum_class_pointer;
 static TrackerDBManagerFlags old_flags = 0;
-
-static GHashTable           *thread_ifaces = NULL; /* Needed for cross-thread cancellation */
-static GStaticMutex          thread_ifaces_mutex = G_STATIC_MUTEX_INIT;
+static guint                 s_cache_size;
+static guint                 u_cache_size;
 
 static GStaticPrivate        interface_data_key = G_STATIC_PRIVATE_INIT;
 
@@ -179,58 +173,6 @@ location_to_directory (TrackerDBLocation location)
 	};
 }
 
-static void
-load_sql_file (TrackerDBInterface *iface,
-               const gchar        *file,
-               const gchar        *delimiter)
-{
-	gchar *path, *content, **queries;
-	gint   count;
-	gint   i;
-
-	path = g_build_filename (sql_dir, file, NULL);
-
-	if (!delimiter) {
-		delimiter = ";";
-	}
-
-	if (!g_file_get_contents (path, &content, NULL, NULL)) {
-		g_critical ("Cannot read SQL file:'%s', please reinstall tracker"
-		            " or check read permissions on the file if it exists", path);
-		g_assert_not_reached ();
-	}
-
-	queries = g_strsplit (content, delimiter, -1);
-
-	for (i = 0, count = 0; queries[i]; i++) {
-		GError *error = NULL;
-		gchar  *sql;
-
-		/* Skip white space, including control characters */
-		for (sql = queries[i]; sql && g_ascii_isspace (sql[0]); sql++);
-
-		if (!sql || sql[0] == '\0') {
-			continue;
-		}
-
-		tracker_db_interface_execute_query (iface, &error, "%s", sql);
-
-		if (error) {
-			g_warning ("Error loading query:'%s' #%d, %s", file, i, error->message);
-			g_error_free (error);
-			continue;
-		}
-
-		count++;
-	}
-
-	g_message ("  Loaded SQL file:'%s' (%d queries)", file, count);
-
-	g_strfreev (queries);
-	g_free (content);
-	g_free (path);
-}
-
 static gboolean
 db_exec_no_reply (TrackerDBInterface *iface,
                   const gchar        *query,
@@ -251,8 +193,14 @@ db_exec_no_reply (TrackerDBInterface *iface,
 }
 
 TrackerDBManagerFlags
-tracker_db_manager_get_flags (void)
+tracker_db_manager_get_flags (guint *select_cache_size, guint *update_cache_size)
 {
+	if (select_cache_size)
+		*select_cache_size = s_cache_size;
+
+	if (update_cache_size)
+		*update_cache_size = u_cache_size;
+
 	return old_flags;
 }
 
@@ -261,17 +209,16 @@ db_set_params (TrackerDBInterface *iface,
                gint                cache_size,
                gint                page_size)
 {
+	TrackerDBResultSet *result_set;
+
 	tracker_db_interface_execute_query (iface, NULL, "PRAGMA synchronous = OFF;");
 	tracker_db_interface_execute_query (iface, NULL, "PRAGMA count_changes = 0;");
 	tracker_db_interface_execute_query (iface, NULL, "PRAGMA temp_store = FILE;");
 	tracker_db_interface_execute_query (iface, NULL, "PRAGMA encoding = \"UTF-8\"");
 	tracker_db_interface_execute_query (iface, NULL, "PRAGMA auto_vacuum = 0;");
 
-	/* if we add direct access library (or enable running queries in parallel with updates,
-	   we need to disable read_uncommitted again
-	   however, when read_uncommitted is disabled, we need the custom page cache (wrapper)
-	   to avoid locking issues between long running batch transactions and queries */
-	tracker_db_interface_execute_query (iface, NULL, "PRAGMA read_uncommitted = True;");
+	result_set = tracker_db_interface_execute_query (iface, NULL, "PRAGMA journal_mode = WAL;");
+	g_object_unref (result_set);
 
 	if (page_size != TRACKER_DB_PAGE_SIZE_DONT_SET) {
 		g_message ("  Setting page size to %d", page_size);
@@ -331,17 +278,6 @@ db_interface_get (TrackerDB  type,
 	return iface;
 }
 
-static TrackerDBInterface *
-db_interface_get_fulltext (void)
-{
-	TrackerDBInterface *iface;
-	gboolean            create;
-
-	iface = db_interface_get (TRACKER_DB_FULLTEXT, &create);
-
-	return iface;
-}
-
 static TrackerDBInterface *
 db_interface_get_metadata (void)
 {
@@ -350,15 +286,6 @@ db_interface_get_metadata (void)
 
 	iface = db_interface_get (TRACKER_DB_METADATA, &create);
 
-	if (create) {
-		tracker_db_interface_start_transaction (iface);
-
-		/* Create tables */
-		load_sql_file (iface, "sqlite-tracker.sql", NULL);
-
-		tracker_db_interface_end_db_transaction (iface);
-	}
-
 	return iface;
 }
 
@@ -372,9 +299,6 @@ db_interface_create (TrackerDB db)
 	case TRACKER_DB_METADATA:
 		return db_interface_get_metadata ();
 
-	case TRACKER_DB_FULLTEXT:
-		return db_interface_get_fulltext ();
-
 	default:
 		g_critical ("This TrackerDB type:%d->'%s' has no interface set up yet!!",
 		            db,
@@ -459,22 +383,44 @@ db_manager_remove_all (gboolean rm_journal)
 	 * calculate the absolute directories here.
 	 */
 	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
+		gchar *filename;
+
 		g_message ("  Removing database:'%s'", dbs[i].abs_filename);
 		g_unlink (dbs[i].abs_filename);
+
+		/* also delete shm and wal helper files */
+		filename = g_strdup_printf ("%s-shm", dbs[i].abs_filename);
+		g_unlink (filename);
+		g_free (filename);
+
+		filename = g_strdup_printf ("%s-wal", dbs[i].abs_filename);
+		g_unlink (filename);
+		g_free (filename);
 	}
 
 	if (rm_journal) {
-		gchar *filename;
+
 
 		db_manager_remove_journal ();
 
 		/* If also the journal is gone, we can also remove db-version.txt, it
 		 * would have no more relevance whatsoever. */
-		filename = g_build_filename (data_dir, TRACKER_DB_VERSION_FILE, NULL);
-		g_message ("  Removing db-version file:'%s'", filename);
-		g_unlink (filename);
-		g_free (filename);
+		db_remove_version_file ();
 	}
+
+	/* Remove locale file also */
+	db_remove_locale_file ();
+}
+
+static void
+db_remove_version_file (void)
+{
+	gchar *filename;
+
+	filename = g_build_filename (data_dir, TRACKER_DB_VERSION_FILE, NULL);
+	g_message ("  Removing db-version file:'%s'", filename);
+	g_unlink (filename);
+	g_free (filename);
 }
 
 static TrackerDBVersion
@@ -538,6 +484,113 @@ db_set_version (void)
 	g_free (filename);
 }
 
+static void
+db_remove_locale_file (void)
+{
+	gchar *filename;
+
+	filename = g_build_filename (data_dir, TRACKER_DB_LOCALE_FILE, NULL);
+	g_message ("  Removing db-locale file:'%s'", filename);
+	g_unlink (filename);
+	g_free (filename);
+}
+
+static gchar *
+db_get_locale (void)
+{
+	gchar *locale = NULL;
+	gchar *filename;
+
+	filename = g_build_filename (data_dir, TRACKER_DB_LOCALE_FILE, NULL);
+
+	if (G_LIKELY (g_file_test (filename, G_FILE_TEST_EXISTS))) {
+		gchar *contents;
+
+		/* Check locale is correct */
+		if (G_LIKELY (g_file_get_contents (filename, &contents, NULL, NULL))) {
+			if (contents && strlen (contents) == 0) {
+				g_critical ("  Empty locale file found at '%s'", filename);
+				g_free (contents);
+			} else {
+				/* Re-use contents */
+				locale = contents;
+			}
+		} else {
+			g_critical ("  Could not get content of file '%s'", filename);
+		}
+	} else {
+		g_critical ("  Could not find database locale file:'%s'", filename);
+	}
+
+	g_free (filename);
+
+	return locale;
+}
+
+static void
+db_set_locale (const gchar *locale)
+{
+	GError *error = NULL;
+	gchar  *filename;
+	gchar  *str;
+
+	filename = g_build_filename (data_dir, TRACKER_DB_LOCALE_FILE, NULL);
+	g_message ("  Creating locale file '%s'", filename);
+
+	str = g_strdup_printf ("%s", locale ? locale : "");
+
+	if (!g_file_set_contents (filename, str, -1, &error)) {
+		g_message ("  Could not set file contents, %s",
+		           error ? error->message : "no error given");
+		g_clear_error (&error);
+	}
+
+	g_free (str);
+	g_free (filename);
+}
+
+gboolean
+tracker_db_manager_locale_changed (void)
+{
+	gchar *db_locale;
+	gchar *current_locale;
+	gboolean changed;
+
+	/* Get current collation locale */
+	current_locale = setlocale (LC_COLLATE, NULL);
+
+	/* Get db locale */
+	db_locale = db_get_locale ();
+
+	/* If they are different, recreate indexes. Note that having
+	 * both to NULL is actually valid, they would default to
+	 * the unicode collation without locale-specific stuff. */
+	if (g_strcmp0 (db_locale, current_locale) != 0) {
+		g_message ("Locale change detected from '%s' to '%s'...",
+		           db_locale, current_locale);
+		changed = TRUE;
+	} else {
+		g_message ("Current and DB locales match: '%s'", db_locale);
+		changed = FALSE;
+	}
+
+	g_free (db_locale);
+	return changed;
+}
+
+void
+tracker_db_manager_set_current_locale (void)
+{
+	const gchar *current_locale;
+
+	/* Get current collation locale */
+	current_locale = setlocale (LC_COLLATE, NULL);
+
+	g_message ("Changing db locale to: '%s'", current_locale);
+
+	db_set_locale (current_locale);
+}
+
 static void
 db_manager_analyze (TrackerDB db)
 {
@@ -602,6 +655,9 @@ db_recreate_all (void)
 		g_object_unref (dbs[i].iface);
 		dbs[i].iface = NULL;
 	}
+
+	/* Initialize locale file */
+	db_set_locale (setlocale (LC_COLLATE, NULL));
 }
 
 void
@@ -632,33 +688,12 @@ tracker_db_manager_init_locations (void)
 	locations_initialized = TRUE;
 }
 
-static void
-free_thread_interface (gpointer data)
-{
-	TrackerDBInterface *interface = data;
-	GHashTableIter iter;
-	gpointer value;
-
-	g_static_mutex_lock (&thread_ifaces_mutex);
-
-	g_hash_table_iter_init (&iter, thread_ifaces);
-
-	while (g_hash_table_iter_next (&iter, NULL, &value)) {
-		if (value == data) {
-			g_hash_table_iter_remove (&iter);
-			break;
-		}
-	}
-
-	g_static_mutex_unlock (&thread_ifaces_mutex);
-
-	g_object_unref (interface);
-}
-
 gboolean
 tracker_db_manager_init (TrackerDBManagerFlags  flags,
                          gboolean              *first_time,
-                         gboolean               shared_cache)
+                         gboolean               shared_cache,
+                         guint                  select_cache_size,
+                         guint                  update_cache_size)
 {
 	GType               etype;
 	TrackerDBVersion    version;
@@ -667,7 +702,6 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 	const gchar        *env_path;
 	gboolean            need_reindex;
 	guint               i;
-	gchar              *in_use_filename;
 	int                 in_use_file;
 	gboolean            loaded = FALSE;
 	TrackerDBInterface *resources_iface;
@@ -701,8 +735,7 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 	old_flags = flags;
 
 	filename = g_strdup_printf ("tracker-%s", g_get_user_name ());
-	if (sys_tmp_dir)
-		g_free (sys_tmp_dir);
+        g_free (sys_tmp_dir);
 	sys_tmp_dir = g_build_filename (g_get_tmp_dir (), filename, NULL);
 	g_free (filename);
 
@@ -716,21 +749,23 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 		sql_dir = g_strdup (env_path);
 	}
 
-	if (user_data_dir)
-		g_free (user_data_dir);
-
+        g_free (user_data_dir);
 	user_data_dir = g_build_filename (g_get_user_data_dir (),
 	                                  "tracker",
 	                                  "data",
 	                                  NULL);
 
-	if (data_dir)
-		g_free (data_dir);
-
+	g_free (data_dir);
 	data_dir = g_build_filename (g_get_user_cache_dir (),
 	                             "tracker",
 	                             NULL);
 
+	in_use_filename = g_build_filename (g_get_user_data_dir (),
+	                                    "tracker",
+	                                    "data",
+	                                    IN_USE_FILENAME,
+	                                    NULL);
+
 	/* Make sure the directories exist */
 	g_message ("Checking database directories exist");
 
@@ -759,8 +794,8 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
 		/* Fill absolute path for the database */
 		dir = location_to_directory (dbs[i].location);
-		if (dbs[i].abs_filename)
-			g_free (dbs[i].abs_filename);
+
+                g_free (dbs[i].abs_filename);
 		dbs[i].abs_filename = g_build_filename (dir, dbs[i].file, NULL);
 
 		/* Check we have each database in place, if one is
@@ -797,12 +832,6 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 		tracker_db_interface_sqlite_enable_shared_cache ();
 	}
 
-	in_use_filename = g_build_filename (g_get_user_data_dir (),
-	                                    "tracker",
-	                                    "data",
-	                                    IN_USE_FILENAME,
-	                                    NULL);
-
 	/* Should we reindex? If so, just remove all databases files,
 	 * NOT the paths, note, that these paths are also used for
 	 * other things like the nfs lock file.
@@ -876,7 +905,7 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 
 				loaded = TRUE;
 
-				stmt = tracker_db_interface_create_statement (dbs[i].iface, NULL,
+				stmt = tracker_db_interface_create_statement (dbs[i].iface, TRACKER_DB_STATEMENT_CACHE_TYPE_NONE, NULL,
 				                                              "PRAGMA integrity_check(1)");
 
 				if (stmt) {
@@ -927,7 +956,6 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 
 	if ((flags & TRACKER_DB_MANAGER_READONLY) == 0) {
 		/* do not create in-use file for read-only mode (direct access) */
-
 		in_use_file = g_open (in_use_filename,
 			              O_WRONLY | O_APPEND | O_CREAT | O_SYNC,
 			              S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
@@ -938,27 +966,28 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 		}
 	}
 
-	g_free (in_use_filename);
-
 	initialized = TRUE;
 
-	thread_ifaces = g_hash_table_new (NULL, NULL);
-
 	if (flags & TRACKER_DB_MANAGER_READONLY) {
-		resources_iface = tracker_db_manager_get_db_interfaces_ro (2,
-		                                                           TRACKER_DB_METADATA,
-		                                                           TRACKER_DB_FULLTEXT);
+		resources_iface = tracker_db_manager_get_db_interfaces_ro (1,
+		                                                           TRACKER_DB_METADATA);
 	} else {
-		resources_iface = tracker_db_manager_get_db_interfaces (2,
-		                                                        TRACKER_DB_METADATA,
-		                                                        TRACKER_DB_FULLTEXT);
+		resources_iface = tracker_db_manager_get_db_interfaces (1,
+		                                                        TRACKER_DB_METADATA);
 	}
 
-	g_static_private_set (&interface_data_key, resources_iface, free_thread_interface);
+	tracker_db_interface_set_max_stmt_cache_size (resources_iface,
+	                                              TRACKER_DB_STATEMENT_CACHE_TYPE_SELECT,
+	                                              select_cache_size);
 
-	g_static_mutex_lock (&thread_ifaces_mutex);
-	g_hash_table_insert (thread_ifaces, g_thread_self (), resources_iface);
-	g_static_mutex_unlock (&thread_ifaces_mutex);
+	tracker_db_interface_set_max_stmt_cache_size (resources_iface,
+	                                              TRACKER_DB_STATEMENT_CACHE_TYPE_UPDATE,
+	                                              update_cache_size);
+
+	s_cache_size = select_cache_size;
+	u_cache_size = update_cache_size;
+
+	g_static_private_set (&interface_data_key, resources_iface, (GDestroyNotify) g_object_unref);
 
 	return TRUE;
 }
@@ -992,22 +1021,9 @@ tracker_db_manager_shutdown (void)
 	sys_tmp_dir = NULL;
 	g_free (sql_dir);
 
-#if HAVE_TRACKER_FTS
-	/* shutdown fts in all threads
-	 * needs to be done before shutting down all db interfaces as
-	 * shutdown does not happen in thread where interface was created
-	 */
-	tracker_fts_shutdown_all ();
-#endif
-
 	/* shutdown db interfaces in all threads */
 	g_static_private_free (&interface_data_key);
 
-	if (thread_ifaces) {
-		g_hash_table_destroy (thread_ifaces);
-		thread_ifaces = NULL;
-	}
-
 	/* Since we don't reference this enum anywhere, we do
 	 * it here to make sure it exists when we call
 	 * g_type_class_peek(). This wouldn't be necessary if
@@ -1023,21 +1039,13 @@ tracker_db_manager_shutdown (void)
 	initialized = FALSE;
 	locations_initialized = FALSE;
 
-	if ((tracker_db_manager_get_flags () & TRACKER_DB_MANAGER_READONLY) == 0) {
+	if ((tracker_db_manager_get_flags (NULL, NULL) & TRACKER_DB_MANAGER_READONLY) == 0) {
 		/* do not delete in-use file for read-only mode (direct access) */
-
-		gchar *in_use_filename;
-
-		in_use_filename = g_build_filename (g_get_user_data_dir (),
-			                            "tracker",
-			                            "data",
-			                            IN_USE_FILENAME,
-			                            NULL);
-
 		g_unlink (in_use_filename);
-
-		g_free (in_use_filename);
 	}
+
+	g_free (in_use_filename);
+	in_use_filename = NULL;
 }
 
 void
@@ -1393,17 +1401,21 @@ tracker_db_manager_get_db_interface (void)
 
 	/* Ensure the interface is there */
 	if (!interface) {
-		interface = tracker_db_manager_get_db_interfaces (2,
-		                                                  TRACKER_DB_METADATA,
-		                                                  TRACKER_DB_FULLTEXT);
+		interface = tracker_db_manager_get_db_interfaces (1,
+		                                                  TRACKER_DB_METADATA);
 
 		tracker_db_interface_sqlite_fts_init (interface, FALSE);
 
-		g_static_private_set (&interface_data_key, interface, free_thread_interface);
 
-		g_static_mutex_lock (&thread_ifaces_mutex);
-		g_hash_table_insert (thread_ifaces, g_thread_self (), interface);
-		g_static_mutex_unlock (&thread_ifaces_mutex);
+		tracker_db_interface_set_max_stmt_cache_size (interface,
+		                                              TRACKER_DB_STATEMENT_CACHE_TYPE_SELECT,
+		                                              s_cache_size);
+
+		tracker_db_interface_set_max_stmt_cache_size (interface,
+		                                              TRACKER_DB_STATEMENT_CACHE_TYPE_UPDATE,
+		                                              u_cache_size);
+
+		g_static_private_set (&interface_data_key, interface, (GDestroyNotify) g_object_unref);
 	}
 
 	return interface;
