@@ -42,7 +42,7 @@
 #include <libtracker-common/tracker-type-utils.h>
 #include <libtracker-common/tracker-utils.h>
 
-#include <libtracker-data/tracker-data.h>
+#include <libtracker-data/tracker-db-manager.h>
 
 #include "tracker-miner-files.h"
 #include "tracker-config.h"
@@ -72,11 +72,7 @@ struct ProcessFileData {
 	TrackerSparqlBuilder *sparql;
 	GCancellable *cancellable;
 	GFile *file;
-#ifdef HAVE_DBUS_FD_PASSING
 	DBusPendingCall *call;
-#else /* HAVE_DBUS_FD_PASSING */
-	DBusGProxyCall *call;
-#endif /* HAVE_DBUS_FD_PASSING */
 };
 
 typedef struct {
@@ -191,6 +187,10 @@ static gboolean    miner_files_process_file             (TrackerMinerFS       *f
                                                          GFile                *file,
                                                          TrackerSparqlBuilder *sparql,
                                                          GCancellable         *cancellable);
+static gboolean    miner_files_process_file_attributes  (TrackerMinerFS       *fs,
+                                                         GFile                *file,
+                                                         TrackerSparqlBuilder *sparql,
+                                                         GCancellable         *cancellable);
 static gboolean    miner_files_monitor_directory        (TrackerMinerFS       *fs,
                                                          GFile                *file);
 static gboolean    miner_files_ignore_next_update_file  (TrackerMinerFS       *fs,
@@ -237,6 +237,7 @@ tracker_miner_files_class_init (TrackerMinerFilesClass *klass)
 	miner_fs_class->check_directory_contents = miner_files_check_directory_contents;
 	miner_fs_class->monitor_directory = miner_files_monitor_directory;
 	miner_fs_class->process_file = miner_files_process_file;
+	miner_fs_class->process_file_attributes = miner_files_process_file_attributes;
 	miner_fs_class->ignore_next_update_file = miner_files_ignore_next_update_file;
         miner_fs_class->finished = miner_files_finished;
 
@@ -658,12 +659,11 @@ set_up_mount_point_cb (GObject      *source,
                        GAsyncResult *result,
                        gpointer      user_data)
 {
+	TrackerSparqlConnection *connection = TRACKER_SPARQL_CONNECTION (source);
 	gchar *removable_device_urn = user_data;
 	GError *error = NULL;
 
-	tracker_miner_execute_update_finish (TRACKER_MINER (source),
-	                                     result,
-	                                     &error);
+	tracker_sparql_connection_update_finish (connection, result, &error);
 
 	if (error) {
 		g_critical ("Could not set mount point in database '%s', %s",
@@ -801,11 +801,12 @@ set_up_mount_point (TrackerMinerFiles *miner,
 	if (accumulator) {
 		g_string_append_printf (accumulator, "%s ", queries->str);
 	} else {
-		tracker_miner_execute_update (TRACKER_MINER (miner),
-		                              queries->str,
-		                              NULL,
-		                              set_up_mount_point_cb,
-		                              g_strdup (removable_device_urn));
+		tracker_sparql_connection_update_async (tracker_miner_get_connection (TRACKER_MINER (miner)),
+                                                        queries->str,
+                                                        G_PRIORITY_DEFAULT,
+                                                        NULL,
+                                                        set_up_mount_point_cb,
+                                                        g_strdup (removable_device_urn));
 	}
 
 	g_string_free (queries, TRUE);
@@ -818,7 +819,7 @@ init_mount_points_cb (GObject      *source,
 {
 	GError *error = NULL;
 
-	tracker_miner_execute_update_finish (TRACKER_MINER (source),
+	tracker_sparql_connection_update_finish (TRACKER_SPARQL_CONNECTION (source),
 	                                     result,
 	                                     &error);
 
@@ -831,7 +832,7 @@ init_mount_points_cb (GObject      *source,
 		 * the mount points, as we need the correct tracker:isMounted value
 		 * for all currently mounted volumes
 		 */
-		init_stale_volume_removal (TRACKER_MINER_FILES (source));
+		init_stale_volume_removal (TRACKER_MINER_FILES (user_data));
 	}
 }
 
@@ -845,20 +846,18 @@ init_mount_points (TrackerMinerFiles *miner_files)
 	gpointer key, value;
 	GString *accumulator;
 	GError *error = NULL;
-	TrackerResultIterator *iterator;
+	TrackerSparqlCursor *cursor;
 	GSList *uuids, *u;
 
 	g_debug ("Initializing mount points...");
 
 	/* First, get all mounted volumes, according to tracker-store (SYNC!) */
-	iterator = tracker_miner_execute_sparql_sync (TRACKER_MINER (miner),
-	                                              "SELECT ?v WHERE { ?v a tracker:Volume ; tracker:isMounted true }",
-	                                              &error);
+	cursor = tracker_sparql_connection_query (tracker_miner_get_connection (miner),
+	                                          "SELECT ?v WHERE { ?v a tracker:Volume ; tracker:isMounted true }",
+	                                          NULL, &error);
 	if (error) {
 		g_critical ("Could not obtain the mounted volumes: %s", error->message);
 		g_error_free (error);
-		if (iterator)
-			tracker_result_iterator_free (iterator);
 		return;
 	}
 
@@ -875,13 +874,13 @@ init_mount_points (TrackerMinerFiles *miner_files)
                              g_strdup (TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN),
                              GINT_TO_POINTER (VOLUME_MOUNTED));
 
-	while (tracker_result_iterator_next (iterator)) {
+	while (tracker_sparql_cursor_next (cursor, NULL, NULL)) {
 		gint state;
 		const gchar *urn;
 
 		state = VOLUME_MOUNTED_IN_STORE;
 
-		urn = tracker_result_iterator_value (iterator, 0);
+		urn = tracker_sparql_cursor_get_string (cursor, 0, NULL);
 
 		if (strcmp (urn, TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN) == 0) {
 			/* Report non-removable media to be mounted by HAL as well */
@@ -891,7 +890,7 @@ init_mount_points (TrackerMinerFiles *miner_files)
 		g_hash_table_replace (volumes, g_strdup (urn), GINT_TO_POINTER (state));
 	}
 
-	tracker_result_iterator_free (iterator);
+	g_object_unref (cursor);
 
 	/* Then, get all currently mounted non-REMOVABLE volumes, according to GIO */
 	uuids = tracker_storage_get_device_uuids (priv->storage, 0, TRUE);
@@ -909,6 +908,9 @@ init_mount_points (TrackerMinerFiles *miner_files)
 		g_hash_table_replace (volumes, non_removable_device_urn, GINT_TO_POINTER (state));
 	}
 
+	g_slist_foreach (uuids, (GFunc) g_free, NULL);
+	g_slist_free (uuids);
+
 	/* Then, get all currently mounted REMOVABLE volumes, according to GIO */
 	uuids = tracker_storage_get_device_uuids (priv->storage, TRACKER_STORAGE_REMOVABLE, FALSE);
 	for (u = uuids; u; u = u->next) {
@@ -994,11 +996,12 @@ init_mount_points (TrackerMinerFiles *miner_files)
 	}
 
 	if (accumulator->str[0] != '\0') {
-		tracker_miner_execute_update (miner,
-		                              accumulator->str,
-		                              NULL,
-		                              init_mount_points_cb,
-		                              NULL);
+		tracker_sparql_connection_update_async (tracker_miner_get_connection (miner),
+                                                        accumulator->str,
+                                                        G_PRIORITY_DEFAULT,
+                                                        NULL,
+                                                        init_mount_points_cb,
+                                                        miner);
 	} else {
 		/* If no further mount point initialization was needed,
 		 * initialize stale volume removal here. */
@@ -1158,11 +1161,12 @@ mount_point_added_cb (TrackerStorage *storage,
 	queries = g_string_new ("");
 	set_up_mount_point (miner, urn, mount_point, TRUE, queries);
 	set_up_mount_point_type (miner, urn, removable, optical, queries);
-	tracker_miner_execute_update (TRACKER_MINER (miner),
-	                              queries->str,
-	                              NULL,
-	                              set_up_mount_point_cb,
-	                              g_strdup (urn));
+	tracker_sparql_connection_update_async (tracker_miner_get_connection (TRACKER_MINER (miner)),
+                                                queries->str,
+                                                G_PRIORITY_DEFAULT,
+                                                NULL,
+                                                set_up_mount_point_cb,
+                                                g_strdup (urn));
 	g_string_free (queries, TRUE);
 	g_free (urn);
 }
@@ -1959,11 +1963,6 @@ extractor_get_embedded_metadata_cb (DBusGProxy *proxy,
 	tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), data->file, NULL);
 
 	process_file_data_free (data);
-#ifndef HAVE_DBUS_FD_PASSING
-	/* When using DBus FD passing, we let the caller free */
-	g_free (preupdate);
-	g_free (sparql);
-#endif /* HAVE_DBUS_FD_PASSING */
 }
 
 static void
@@ -1973,10 +1972,6 @@ extractor_get_embedded_metadata_cancel (GCancellable    *cancellable,
 	GError *error;
 
 	/* Cancel extractor call */
-#ifndef HAVE_DBUS_FD_PASSING
-	dbus_g_proxy_cancel_call (data->miner->private->extractor_proxy,
-	                          data->call);
-#endif /* HAVE_DBUS_FD_PASSING */
 
 	error = g_error_new_literal (miner_files_error_quark, 0, "Embedded metadata extraction was cancelled");
 	tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), data->file, error);
@@ -1985,7 +1980,6 @@ extractor_get_embedded_metadata_cancel (GCancellable    *cancellable,
 	g_error_free (error);
 }
 
-#ifdef HAVE_DBUS_FD_PASSING
 static FastAsyncData*
 fast_async_data_new (org_freedesktop_Tracker1_Extract_get_metadata_reply callback,
                      gpointer       user_data)
@@ -2008,6 +2002,7 @@ fast_async_data_free (FastAsyncData *data)
 static void
 get_metadata_fast_cb (void     *buffer,
                       gssize    buffer_size,
+                      GStrv     variable_names,
                       GError   *error,
                       gpointer  user_data)
 {
@@ -2080,18 +2075,17 @@ get_metadata_fast_async (DBusConnection  *connection,
 	tracker_dbus_send_and_splice_async (connection,
 	                                    message,
 	                                    pipefd[0],
+	                                    FALSE,
 	                                    cancellable,
 	                                    get_metadata_fast_cb,
 	                                    data);
 }
-#endif /* HAVE_DBUS_FD_PASSING */
 
 static void
 extractor_get_embedded_metadata (ProcessFileData *data,
                                  const gchar     *uri,
                                  const gchar     *mime_type)
 {
-#ifdef HAVE_DBUS_FD_PASSING
 	get_metadata_fast_async (dbus_g_connection_get_connection (data->miner->private->connection),
 	                         uri,
 	                         mime_type,
@@ -2099,13 +2093,7 @@ extractor_get_embedded_metadata (ProcessFileData *data,
 	                         extractor_get_embedded_metadata_cb,
 	                         data);
 	data->call = NULL;
-#else /* HAVE_DBUS_FD_PASSING */
-	data->call = org_freedesktop_Tracker1_Extract_get_metadata_async (data->miner->private->extractor_proxy,
-	                                                                  uri,
-	                                                                  mime_type,
-	                                                                  extractor_get_embedded_metadata_cb,
-	                                                                  data);
-#endif
+
 	g_signal_connect (data->cancellable, "cancelled",
 	                  G_CALLBACK (extractor_get_embedded_metadata_cancel), data);
 }
@@ -2236,6 +2224,119 @@ miner_files_process_file (TrackerMinerFS       *fs,
 	return TRUE;
 }
 
+static void
+process_file_attributes_cb (GObject      *object,
+                            GAsyncResult *result,
+                            gpointer      user_data)
+{
+	TrackerSparqlBuilder *sparql;
+	ProcessFileData *data;
+	const gchar *urn;
+	GFileInfo *file_info;
+	guint64 time_;
+	GFile *file;
+	gchar *uri;
+	GError *error = NULL;
+	gboolean is_iri;
+
+	data = user_data;
+	file = G_FILE (object);
+	sparql = data->sparql;
+	file_info = g_file_query_info_finish (file, result, &error);
+
+	if (error) {
+		/* Something bad happened, notify about the error */
+		tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), file, error);
+		process_file_data_free (data);
+		g_error_free (error);
+		return;
+	}
+
+	uri = g_file_get_uri (file);
+	urn = miner_files_get_file_urn (TRACKER_MINER_FILES (data->miner), file, &is_iri);
+
+	/* We MUST have an IRI in attributes updating */
+	if (!is_iri) {
+		error = g_error_new_literal (miner_files_error_quark,
+		                             0,
+		                             "Received request to update attributes but no IRI available!");
+		/* Notify about the error */
+		tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), file, error);
+		process_file_data_free (data);
+		g_error_free (error);
+		return;
+	}
+
+	/* DELETE old property values */
+	tracker_sparql_builder_delete_open (sparql, TRACKER_MINER_FS_GRAPH_URN);
+	tracker_sparql_builder_subject_iri (sparql, urn);
+	tracker_sparql_builder_predicate (sparql, "nfo:fileLastModified");
+	tracker_sparql_builder_object_variable (sparql, "lastmodified");
+	tracker_sparql_builder_predicate (sparql, "nfo:fileLastAccessed");
+	tracker_sparql_builder_object_variable (sparql, "lastaccessed");
+	tracker_sparql_builder_delete_close (sparql);
+
+	tracker_sparql_builder_where_open (sparql);
+	tracker_sparql_builder_subject_iri (sparql, urn);
+	tracker_sparql_builder_predicate (sparql, "nfo:fileLastModified");
+	tracker_sparql_builder_object_variable (sparql, "lastmodified");
+	tracker_sparql_builder_predicate (sparql, "nfo:fileLastAccessed");
+	tracker_sparql_builder_object_variable (sparql, "lastaccessed");
+	tracker_sparql_builder_where_close (sparql);
+
+	/* INSERT new property values */
+	tracker_sparql_builder_insert_open (sparql, TRACKER_MINER_FS_GRAPH_URN);
+	tracker_sparql_builder_subject_iri (sparql, urn);
+
+	time_ = g_file_info_get_attribute_uint64 (file_info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
+	tracker_sparql_builder_predicate (sparql, "nfo:fileLastModified");
+	tracker_sparql_builder_object_date (sparql, (time_t *) &time_);
+
+	time_ = g_file_info_get_attribute_uint64 (file_info, G_FILE_ATTRIBUTE_TIME_ACCESS);
+	tracker_sparql_builder_predicate (sparql, "nfo:fileLastAccessed");
+	tracker_sparql_builder_object_date (sparql, (time_t *) &time_);
+
+	tracker_sparql_builder_insert_close (sparql);
+
+	g_object_unref (file_info);
+	g_free (uri);
+
+	/* Notify about the success */
+	tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), data->file, NULL);
+
+	process_file_data_free (data);
+}
+
+static gboolean
+miner_files_process_file_attributes (TrackerMinerFS       *fs,
+                                     GFile                *file,
+                                     TrackerSparqlBuilder *sparql,
+                                     GCancellable         *cancellable)
+{
+	ProcessFileData *data;
+	const gchar *attrs;
+
+	data = g_slice_new0 (ProcessFileData);
+	data->miner = g_object_ref (fs);
+	data->cancellable = g_object_ref (cancellable);
+	data->sparql = g_object_ref (sparql);
+	data->file = g_object_ref (file);
+
+	/* Query only attributes that may change in an ATTRIBUTES_UPDATED event */
+	attrs = G_FILE_ATTRIBUTE_TIME_MODIFIED ","
+		G_FILE_ATTRIBUTE_TIME_ACCESS;
+
+	g_file_query_info_async (file,
+	                         attrs,
+	                         G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+	                         G_PRIORITY_DEFAULT,
+	                         cancellable,
+	                         process_file_attributes_cb,
+	                         data);
+
+	return TRUE;
+}
+
 static gboolean
 miner_files_ignore_next_update_file (TrackerMinerFS       *fs,
                                      GFile                *file,
@@ -2576,7 +2677,7 @@ remove_files_in_removable_media_cb (GObject      *object,
 {
 	GError *error = NULL;
 
-	tracker_miner_execute_update_finish (TRACKER_MINER (object), result, &error);
+	tracker_sparql_connection_update_finish (TRACKER_SPARQL_CONNECTION (object), result, &error);
 
 	if (error) {
 		g_critical ("Could not remove files in volumes: %s", error->message);
@@ -2617,11 +2718,12 @@ miner_files_in_removable_media_remove_by_type (TrackerMinerFiles  *miner,
 		                        removable ? "true" : "false",
 		                        optical ? "true" : "false");
 
-		tracker_miner_execute_batch_update (TRACKER_MINER (miner),
-		                                    queries->str,
-		                                    NULL,
-		                                    remove_files_in_removable_media_cb,
-		                                    NULL);
+		tracker_sparql_connection_update_async (tracker_miner_get_connection (TRACKER_MINER (miner)),
+                                                        queries->str,
+                                                        G_PRIORITY_DEFAULT,
+                                                        NULL,
+                                                        remove_files_in_removable_media_cb,
+                                                        NULL);
 
 		g_string_free (queries, TRUE);
 
@@ -2658,11 +2760,12 @@ miner_files_in_removable_media_remove_by_date (TrackerMinerFiles  *miner,
 	                        "}",
 	                        date);
 
-	tracker_miner_execute_batch_update (TRACKER_MINER (miner),
-	                                    queries->str,
-	                                    NULL,
-	                                    remove_files_in_removable_media_cb,
-	                                    NULL);
+	tracker_sparql_connection_update_async (tracker_miner_get_connection (TRACKER_MINER (miner)),
+                                                queries->str,
+                                                G_PRIORITY_DEFAULT,
+                                                NULL,
+                                                remove_files_in_removable_media_cb,
+                                                NULL);
 
 	g_string_free (queries, TRUE);
 }
