@@ -32,8 +32,6 @@
 #include "tracker-dbus.h"
 #include "tracker-resources.h"
 #include "tracker-resources-glue.h"
-#include "tracker-resource-class.h"
-#include "tracker-resources-class-glue.h"
 #include "tracker-status.h"
 #include "tracker-status-glue.h"
 #include "tracker-statistics.h"
@@ -41,19 +39,14 @@
 #include "tracker-backup.h"
 #include "tracker-backup-glue.h"
 #include "tracker-marshal.h"
-
-#ifdef HAVE_DBUS_FD_PASSING
 #include "tracker-steroids.h"
-#endif
 
 static DBusGConnection *connection;
 static DBusGProxy      *gproxy;
 static GSList          *objects;
 static TrackerStatus   *notifier;
 static TrackerBackup   *backup;
-#ifdef HAVE_DBUS_FD_PASSING
 static TrackerSteroids *steroids;
-#endif
 
 static gboolean
 dbus_register_service (DBusGProxy  *proxy,
@@ -187,32 +180,31 @@ dbus_set_available (gboolean available)
 			tracker_dbus_register_objects ();
 		}
 	} else {
-                GSList *l;
+		GSList *l;
 
-                if (objects) {
-                        dbus_g_proxy_disconnect_signal (gproxy,
-                                                        "NameOwnerChanged",
-                                                        G_CALLBACK (name_owner_changed_cb),
-                                                        tracker_dbus_get_object (TRACKER_TYPE_RESOURCES));
-                }
+		if (objects) {
+			dbus_g_proxy_disconnect_signal (gproxy,
+			                                "NameOwnerChanged",
+			                                G_CALLBACK (name_owner_changed_cb),
+			                                tracker_dbus_get_object (TRACKER_TYPE_RESOURCES));
+		}
 
-#ifdef HAVE_DBUS_FD_PASSING
-                if (steroids) {
+		if (steroids) {
 			dbus_connection_remove_filter (dbus_g_connection_get_connection (connection),
 			                               tracker_steroids_connection_filter,
 			                               steroids);
 			g_object_unref (steroids);
 			steroids = NULL;
-                }
-#endif
+		}
 
-                for (l = objects; l; l = l->next) {
-                        dbus_g_connection_unregister_g_object (connection, l->data);
-                        g_object_unref (l->data);
-                }
 
-                g_slist_free (objects);
-                objects = NULL;
+		for (l = objects; l; l = l->next) {
+			dbus_g_connection_unregister_g_object (connection, l->data);
+			g_object_unref (l->data);
+		}
+
+		g_slist_free (objects);
+		objects = NULL;
 	}
 }
 
@@ -288,7 +280,7 @@ tracker_dbus_register_objects (void)
 	objects = g_slist_prepend (objects, object);
 
 	/* Add org.freedesktop.Tracker1.Resources */
-	object = resources = tracker_resources_new ();
+	object = resources = tracker_resources_new (connection);
 	if (!object) {
 		g_critical ("Could not create TrackerResources object to register");
 		return FALSE;
@@ -306,7 +298,6 @@ tracker_dbus_register_objects (void)
 	                      TRACKER_RESOURCES_PATH);
 	objects = g_slist_prepend (objects, object);
 
-#ifdef HAVE_DBUS_FD_PASSING
 	if (!steroids) {
 		/* Add org.freedesktop.Tracker1.Steroids */
 		steroids = tracker_steroids_new ();
@@ -322,7 +313,6 @@ tracker_dbus_register_objects (void)
 		/* Note: TrackerSteroids should not go to the 'objects' list, as it is
 		 * a filter, not an object registered */
 	}
-#endif
 
 	/* Reverse list since we added objects at the top each time */
 	objects = g_slist_reverse (objects);
@@ -348,102 +338,18 @@ tracker_dbus_register_objects (void)
 }
 
 gboolean
-tracker_dbus_register_class_signal_objects (void)
+tracker_dbus_register_prepare_class_signal (void)
 {
-	TrackerDBResultSet *result_set;
-	GSList *event_sources = NULL;
-	GStrv classes, p;
-	gpointer object, resources;
+	gpointer resources;
 
 	resources = tracker_dbus_get_object (TRACKER_TYPE_RESOURCES);
 
 	if (!resources) {
 		g_message ("Error during initialization, Resources DBus object not available");
-		return TRUE;
+		return FALSE;
 	}
 
-	result_set = tracker_data_query_sparql ("SELECT ?class WHERE { ?class tracker:notify true }", NULL);
-
-	if (!result_set) {
-		g_message ("No Nepomuk classes to register on D-Bus");
-		return TRUE;
-	}
-
-	classes = tracker_dbus_query_result_to_strv (result_set, 0, NULL);
-	g_object_unref (result_set);
-
-	if (!classes) {
-		g_message ("No Nepomuk classes to register on D-Bus");
-		return TRUE;
-	}
-
-	for (p = classes; *p; p++) {
-		TrackerNamespace *namespace;
-		const gchar *rdf_class;
-		gchar *namespace_uri;
-		gchar *replaced, *path, *hash;
-
-		rdf_class = *p;
-		hash = strrchr (rdf_class, '#');
-
-		if (!hash) {
-			/* Support ontologies whose namespace
-			 * uri does not end in a hash, e.g.
-			 * dc.
-			 */
-			hash = strrchr (rdf_class, '/');
-		}
-
-		if (!hash) {
-			g_critical ("Unknown namespace for class:'%s'",
-			            rdf_class);
-			continue;
-		}
-
-		namespace_uri = g_strndup (rdf_class, hash - rdf_class + 1);
-		namespace = tracker_ontologies_get_namespace_by_uri (namespace_uri);
-		g_free (namespace_uri);
-
-		if (!namespace) {
-			g_critical ("Unknown namespace:'%s' for class:'%s'",
-			            namespace_uri,
-			            rdf_class);
-			continue;
-		}
-
-		replaced = g_strdup_printf ("%s/%s",
-		                            tracker_namespace_get_prefix (namespace),
-		                            hash + 1);
-		path = g_strdup_printf (TRACKER_RESOURCES_CLASS_PATH,
-		                        replaced);
-		g_free (replaced);
-
-		/* Add a org.freedesktop.Tracker1.Resources.Class */
-		object = tracker_resource_class_new (rdf_class, path, connection);
-		if (!object) {
-			g_critical ("Could not create TrackerResourcesClass object to register:'%s' class",
-			            rdf_class);
-			g_free (path);
-			return FALSE;
-		}
-
-		dbus_register_object (connection,
-		                      gproxy,
-		                      G_OBJECT (object),
-		                      &dbus_glib_tracker_resources_class_object_info,
-		                      path);
-		g_free (path);
-
-		/* TrackerResources takes over ownership and unrefs
-		 * the gobjects too.
-		 */
-		event_sources = g_slist_prepend (event_sources, g_object_ref (object));
-		objects = g_slist_prepend (objects, object);
-	}
-
-	g_strfreev (classes);
-
-	tracker_resources_prepare (resources, event_sources);
+	tracker_resources_prepare (resources);
 
 	return TRUE;
 }
@@ -459,11 +365,9 @@ tracker_dbus_get_object (GType type)
 		}
 	}
 
-#ifdef HAVE_DBUS_FD_PASSING
 	if (steroids && type == TRACKER_TYPE_STEROIDS) {
 		return G_OBJECT (steroids);
 	}
-#endif /* HAVE_DBUS_FD_PASSING */
 
 	if (notifier && type == TRACKER_TYPE_STATUS) {
 		return G_OBJECT (notifier);
